﻿#include "CWindow.h"
#include <dsound.h>

#define DEBUGMODE
#include "DEBUG.H"

// ライブラリリンク
#pragma comment(lib, "dsound.lib")
#pragma comment(lib, "winmm.lib")


// グローバル変数
CWindow				win;					// ウインドウクラス
LPDIRECTSOUND8		lpDS = NULL;			// DirectSound8
LPDIRECTSOUNDBUFFER	lpPrimary = NULL;		// プライマリサウンドバッファ
LPDIRECTSOUNDBUFFER	lpSecondary = NULL;		// セカンダリサウンドバッファ



/////////////////////////////////////////////////////////
// 初期化
/////////////////////////////////////////////////////////
BOOL Init( void )
{
	HRESULT ret;

	// COMの初期化
	CoInitialize( NULL );

	// DirectSound8を作成
	ret = DirectSoundCreate8( NULL,&lpDS,NULL );
	if( FAILED(ret) ) {
		DEBUG( "サウンドオブジェクト作成失敗\n" );
		return FALSE;
	}

	// 強調モード
	ret = lpDS->SetCooperativeLevel( win.hWnd,DSSCL_EXCLUSIVE|DSSCL_PRIORITY );
	if( FAILED(ret) ) {
		DEBUG( "強調レベル設定失敗\n" );
		return FALSE;
	}

	return TRUE;
}

/////////////////////////////////////////////////////////
// プライマリサウンドバッファの作成
/////////////////////////////////////////////////////////
BOOL CreatePrimaryBuffer( void )
{
	HRESULT ret;
	WAVEFORMATEX wf;

	// プライマリサウンドバッファの作成
	DSBUFFERDESC dsdesc;
	ZeroMemory( &dsdesc,sizeof(DSBUFFERDESC) );
	dsdesc.dwSize			= sizeof( DSBUFFERDESC );
	dsdesc.dwFlags			= DSBCAPS_PRIMARYBUFFER;
	dsdesc.dwBufferBytes	= 0;
	dsdesc.lpwfxFormat		= NULL;
	ret = lpDS->CreateSoundBuffer( &dsdesc,&lpPrimary,NULL );
	if( FAILED(ret) ) {
		DEBUG( "プライマリサウンドバッファ作成失敗\n" );
		return FALSE;
	}

	// プライマリバッファのステータスを決定
	wf.cbSize = sizeof( WAVEFORMATEX );
	wf.wFormatTag			= WAVE_FORMAT_PCM;
	wf.nChannels			= 2;
	wf.nSamplesPerSec		= 44100;
	wf.wBitsPerSample		= 16;
	wf.nBlockAlign			= wf.nChannels * wf.wBitsPerSample / 8;
	wf.nAvgBytesPerSec		= wf.nSamplesPerSec * wf.nBlockAlign;
	ret = lpPrimary->SetFormat( &wf );
	if( FAILED(ret) ) {
		DEBUG( "プライマリバッファのステータス失敗\n" );
		return FALSE;
	}

	return TRUE;
}

/////////////////////////////////////////////////////////
// サウンドバッファの作成
/////////////////////////////////////////////////////////
BOOL CreateSoundBuffer( LPDIRECTSOUNDBUFFER *dsb,const char *file )
{
	HRESULT ret;
	MMCKINFO mSrcWaveFile;
	MMCKINFO mSrcWaveFmt;
	MMCKINFO mSrcWaveData;
	LPWAVEFORMATEX wf;

	// WAVファイルをロード
	HMMIO hSrc;
	hSrc = mmioOpenA( (LPSTR)file,NULL,MMIO_ALLOCBUF|MMIO_READ|MMIO_COMPAT );
	if( !hSrc ) {
		DEBUG( "WAVファイルロードエラー\n" );
		return FALSE;
	}

	// 'WAVE'チャンクチェック
	ZeroMemory( &mSrcWaveFile,sizeof(mSrcWaveFile) );
	ret = mmioDescend( hSrc,&mSrcWaveFile,NULL,MMIO_FINDRIFF );
	if( mSrcWaveFile.fccType!=mmioFOURCC('W','A','V','E') ) {
		DEBUG( "WAVEチャンクチェックエラー\n" );
		mmioClose( hSrc,0 );
		return FALSE;
	}

	// 'fmt 'チャンクチェック
	ZeroMemory( &mSrcWaveFmt,sizeof(mSrcWaveFmt) );
	ret = mmioDescend( hSrc,&mSrcWaveFmt,&mSrcWaveFile,MMIO_FINDCHUNK );
	if( mSrcWaveFmt.ckid!=mmioFOURCC('f','m','t',' ') ) {
		DEBUG( "fmt チャンクチェックエラー\n" );
		mmioClose( hSrc,0 );
		return FALSE;
	}

	// ヘッダサイズの計算
	int iSrcHeaderSize = mSrcWaveFmt.cksize;
	if( iSrcHeaderSize<sizeof(WAVEFORMATEX) )
		iSrcHeaderSize=sizeof(WAVEFORMATEX);

	// ヘッダメモリ確保
	wf = (LPWAVEFORMATEX)malloc( iSrcHeaderSize );
	if( !wf ) {
		DEBUG( "メモリ確保エラー\n" );
		mmioClose( hSrc,0 );
		return FALSE;
	}
	ZeroMemory( wf,iSrcHeaderSize );

	// WAVEフォーマットのロード
	ret = mmioRead( hSrc,(char*)wf,mSrcWaveFmt.cksize );
	if( FAILED(ret) ) {
		DEBUG( "WAVEフォーマットロードエラー\n" );
		free( wf );
		mmioClose( hSrc,0 );
		return FALSE;
	}
	DEBUG( "チャンネル数       = %d\n",wf->nChannels );
	DEBUG( "サンプリングレート = %d\n",wf->nSamplesPerSec );
	DEBUG( "ビットレート       = %d\n",wf->wBitsPerSample );


	// fmtチャンクに戻る
	mmioAscend( hSrc,&mSrcWaveFmt,0 );

	// dataチャンクを探す
	while(1) {
		// 検索
		ret = mmioDescend( hSrc,&mSrcWaveData,&mSrcWaveFile,0 );
		if( FAILED(ret) ) {
			DEBUG( "dataチャンクが見つからない\n" );
			free( wf );
			mmioClose( hSrc,0 );
			return FALSE;
		}
		if( mSrcWaveData.ckid==mmioStringToFOURCCA("data",0) )
			break;
		// 次のチャンクへ
		ret = mmioAscend( hSrc,&mSrcWaveData,0 );
	}
	DEBUG( "データサイズ       = %d\n",mSrcWaveData.cksize );


	// サウンドバッファの作成
	DSBUFFERDESC dsdesc;
	ZeroMemory( &dsdesc,sizeof(DSBUFFERDESC) );
	dsdesc.dwSize = sizeof( DSBUFFERDESC );
	dsdesc.dwFlags = DSBCAPS_GETCURRENTPOSITION2 | DSBCAPS_STATIC | DSBCAPS_LOCDEFER;
	dsdesc.dwBufferBytes = mSrcWaveData.cksize;
	dsdesc.lpwfxFormat = wf;
	dsdesc.guid3DAlgorithm = DS3DALG_DEFAULT;
	ret = lpDS->CreateSoundBuffer( &dsdesc,dsb,NULL );
	if( FAILED(ret) ) {
		DEBUG( "サウンドバッファの作成エラー\n" );
		free( wf );
		mmioClose( hSrc,0 );
		return FALSE;
	}

	// ロック開始
	LPVOID pMem1,pMem2;
	DWORD dwSize1,dwSize2;
	ret = (*dsb)->Lock( 0,mSrcWaveData.cksize,&pMem1,&dwSize1,&pMem2,&dwSize2,0 );
	if( FAILED(ret) ) {
		DEBUG( "ロック失敗\n" );
		free( wf );
		mmioClose( hSrc,0 );
		return FALSE;
	}

	// データ書き込み
	mmioRead( hSrc,(char*)pMem1,dwSize1 );
	mmioRead( hSrc,(char*)pMem2,dwSize2 );

	// ロック解除
	(*dsb)->Unlock( pMem1,dwSize1,pMem2,dwSize2 );

	// ヘッダ用メモリを開放
	free( wf );

	// WAVを閉じる
	mmioClose( hSrc,0 );

	return TRUE;
}

/////////////////////////////////////////////////////////
// 終了
/////////////////////////////////////////////////////////
BOOL Exit( void )
{
	if( lpSecondary ) {
		lpSecondary->Release();
		lpSecondary = NULL;
	}

	if( lpPrimary ) {
		lpPrimary->Release();
		lpPrimary = NULL;
	}

	if( lpDS ) {
		lpDS->Release();
		lpDS = NULL;
	}

	// COMの終了
	CoUninitialize();

	return TRUE;
}

// メインルーチン
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdParam, int nCmdShow)
{
	INITDEBUG();
	CLEARDEBUG();

	// ウインドウ
	if( !win.Create( hInstance,L"TestDirectSound",TRUE,640,480) ) {
		DEBUG( "ウィンドウエラー\n" );
        return -1;
	}

	// DirectSound初期化
	if( !Init() ) {
		Exit();
		return -1;
	}

	// プライマリサウンドバッファ
	if( !CreatePrimaryBuffer() ) {
		Exit();
		return -1;
	}

	// サウンドバッファ
	if( !CreateSoundBuffer(&lpSecondary,"nayu_dra.wav") ) {
		Exit();
		return -1;
	}

	// 起動時の再生
	lpSecondary->Play( 0,0,0 );

	MSG msg;
	while(1) {
		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
			if (msg.message == WM_QUIT) {
				DEBUG( "WM_QUIT\n" );
				break;
			}
			TranslateMessage(&msg);
        	DispatchMessage(&msg);
		}

		BYTE key[256];
		GetKeyboardState( key );

		if( key[VK_ESCAPE]&0x80 )
			break;

		if( key[VK_F1]&0x80 )
			lpSecondary->Play( 0,0,0 );

		if( key[VK_F2]&0x80 )
			lpSecondary->Stop();

		if( key[VK_F3]&0x80 )
			lpSecondary->SetCurrentPosition(0);

		Sleep(8);										// CPUへのウエイト
	}

	// 開放
	Exit();

	win.Delete();

	return 0;
}
